#ifndef TTEXTTOGRAPH
#define TTEXTTOGRAPH

#include <TFieldSelection.h>
// ROOT headers
#include <TGraph.h>
#include <TGraphErrors.h>
#include <TGraphAsymmErrors.h>
/// STL headers
#include <map>
#include <stdexcept>

/** \class TTextToData
* \brief Class template which simplifies the creation of ROOT graph classes from arbitrary rows and columns
* of a text file.
*
* This class simplifies the creation of ROOT graph classes from arbitrary rows and columns
* of a text file. The majority of the processing is done by the TTextToData and TFieldSelection classes.
*
* \author Hugh Dickinson
*/
template<typename GraphType>
class TTextToGraph : public GraphType {
	
public :
	/// Default constructor
	TTextToGraph():GraphType(){}
	/// Constructor taking filename and comma separated column string.
	TTextToGraph(std::string fileName, TFieldSelector cols, TFieldSelector rows = "");
	/// Destructor
	virtual ~TTextToGraph(){}
	
private :
	/// Parse the input file using TTextToData and fill the graph arrays.
	void ParseFile(std::string fileName, TFieldSelector& cols, TFieldSelector& rows);
	
	ClassDef(TTextToGraph, 1);
};

template<typename GraphType>
TTextToGraph<GraphType>::TTextToGraph(std::string fileName, TFieldSelector cols, TFieldSelector rows):
GraphType()
{
	try{
		ParseFile(fileName, cols, rows);
	}
	catch(std::runtime_error& e){
		std::cerr << "TTextToGraph error: " << e.what() << "." << std::endl;
		throw std::runtime_error("Runtime error in TTextToGraph");
	}
}

typedef TTextToGraph<TGraph> TTTGraph;
typedef TTextToGraph<TGraphErrors> TTTGraphErrors;
typedef TTextToGraph<TGraphAsymmErrors> TTTGraphAsymmErrors;


#endif // TTEXTTOGRAPH
