#include "TFieldSelection.h"

// STL headers.
#include <sstream>
#include <algorithm>
#include <stdexcept>

ClassImp(TFieldSelection);

TFieldSelection::TFieldSelection(std::string fields){
	ParseFields(fields);
}

TFieldSelection::TFieldSelection(const char * fields){
	try{
		ParseFields(std::string(fields));
	}
	catch(std::runtime_error& e){
		std::cerr << "TFieldSelection error: " << e.what() << "." << std::endl;
		throw std::runtime_error("Runtime error in TFieldSelection");
	}
}

void TFieldSelection::ParseFields(std::string fields){
	int field;
	char separator;
	std::stringstream fieldStream;
	fieldStream << fields;
	while(!fieldStream.eof()){
		fieldStream >> field;
		if(fieldStream.fail()){
			throw std::runtime_error("Badly formed field selection string");
		}
		fieldStream >> separator;
		if(fieldStream.eof() || separator == ','){
			fFields.push_back(field);
		}
		else if(separator == '-'){
			int field2;
			fieldStream >> field2;
			if(fieldStream.fail()){
				throw std::runtime_error("Badly formed field selection string");
			}
			for(int iField = field; iField <= field2; iField++){
				fFields.push_back(iField);
			}
			// consume next separator (should be a comma)
			fieldStream >> separator;
			if((!fieldStream.eof()) && separator != ','){
				throw std::runtime_error("Badly formed field selection string");
			}
		}
		else{
			// badly formed field string.
			throw std::runtime_error("Badly formed field selection string");
		}
	}
	if(fFields.empty()){
		throw std::runtime_error("No fields selected");
	}
}

bool TFieldSelection::IsRequiredField(int field){
	return (std::find(fFields.begin(), fFields.end(), field) != fFields.end());
}


TFieldSelector::TFieldSelector(const char * fields):
fFields(fields)
{}

TFieldSelection * TFieldSelector::GetSelection(){
	if(fFields.empty() || fFields == "allFields"){
		return new TSelectAll();
	}
	else{
		return new TFieldSelection(fFields);
	}
}
