#include "TTextToGraph.h"
#include <TTextToData.h>

templateClassImp(TTextToGraph);

template<>
void TTextToGraph<TGraph>::ParseFile(std::string fileName, TFieldSelector& cols, TFieldSelector& rows){
	TTextToDoubles ttd(fileName, cols, rows);
	TFieldSelection * colSel = cols.GetSelection();
	const std::vector<int>& orderedCols = colSel->GetFields();	
	std::vector<double> * xVals = ttd.GetCol(orderedCols[0]);
	std::vector<double> * yVals = ttd.GetCol(orderedCols[1]);
	if(xVals == 0 || yVals == 0){
		throw std::runtime_error("Empty column vectors");
	}
	for(int iRow = 0; iRow < xVals->size(); iRow++){
		this->SetPoint(iRow, xVals->at(iRow), yVals->at(iRow));
	}
	delete xVals, yVals;
}

template<>
void TTextToGraph<TGraphErrors>::ParseFile(std::string fileName, TFieldSelector& cols, TFieldSelector& rows){
	TTextToDoubles ttd(fileName, cols, rows);
	TFieldSelection * colSel = cols.GetSelection();
	const std::vector<int>& orderedCols = colSel->GetFields();	
	std::vector<double> * xVals = ttd.GetCol(orderedCols[0]);
	std::vector<double> * yVals = ttd.GetCol(orderedCols[1]);
	std::vector<double> * xErrs = ttd.GetCol(orderedCols[2]);
	std::vector<double> * yErrs = ttd.GetCol(orderedCols[3]);
	if(xVals == 0 || yVals == 0 || xErrs == 0 || yErrs == 0){
		throw std::runtime_error("Empty column vectors");
	}
	for(int iRow = 0; iRow < xVals->size(); iRow++){
		this->SetPoint(iRow, xVals->at(iRow), yVals->at(iRow));
		this->SetPointError(iRow, xErrs->at(iRow), yErrs->at(iRow));
	}
	delete xVals, yVals, xErrs, yErrs;
}

template<>
void TTextToGraph<TGraphAsymmErrors>::ParseFile(std::string fileName, TFieldSelector& cols, TFieldSelector& rows){
	TTextToDoubles ttd(fileName, cols, rows);
	TFieldSelection * colSel = cols.GetSelection();
	const std::vector<int>& orderedCols = colSel->GetFields();
	std::vector<double> * xVals = ttd.GetCol(orderedCols[0]);
	std::vector<double> * yVals = ttd.GetCol(orderedCols[1]);
	std::vector<double> * xErrsLo = ttd.GetCol(orderedCols[2]);
	std::vector<double> * xErrsHi = ttd.GetCol(orderedCols[3]);
	std::vector<double> * yErrsLo = ttd.GetCol(orderedCols[4]);
	std::vector<double> * yErrsHi = ttd.GetCol(orderedCols[5]);
	if(xVals == 0 || yVals == 0 || xErrsLo == 0 || yErrsLo == 0 || xErrsHi == 0 || yErrsHi == 0){
		throw std::runtime_error("Empty column vectors");
	}
	for(int iRow = 0; iRow < xVals->size(); iRow++){
		this->SetPoint(iRow, xVals->at(iRow), yVals->at(iRow));
		this->SetPointEXhigh(iRow, xErrsHi->at(iRow));
		this->SetPointEXlow(iRow, xErrsLo->at(iRow));
		this->SetPointEYhigh(iRow, yErrsHi->at(iRow));
		this->SetPointEYlow(iRow, yErrsLo->at(iRow));
	}
	delete xVals, yVals, xErrsLo, xErrsHi, yErrsLo, yErrsHi;
}
